st7/
;------------------------------------------------------------------------------
; http://st7-mcu.dxp.pl
; File : I2C.asm
; Description : Software implementation of I2C master
; (C) Radoslaw Kwiecien, radek@dxp.pl
;------------------------------------------------------------------------------
		.nolist	
		#INCLUDE "st7flite19.inc"
		.LIST
;------------------------------------------------------------------------------
; I/O configuration. Change this values if need.
;------------------------------------------------------------------------------
		#define	I2C_DIR	PBDDR
		#define I2C_DAT	PBDR
		
		#define	SDA	5
		#define SCL	6
				
;------------------------------------------------------------------------------
; Data RAM space
;------------------------------------------------------------------------------
		BYTES
		segment	'ram0'
i2c_counter	DS.B 1
i2c_data		DS.B 1

;------------------------------------------------------------------------------
; Program Flash space
;------------------------------------------------------------------------------
		WORDS
		SEGMENT	'rom'
		
;------------------------------------------------------------------------------
; Delay routine
;------------------------------------------------------------------------------
.i2c_delay
		NOP
		NOP
		RET
		
;------------------------------------------------------------------------------
; START routine
;------------------------------------------------------------------------------
.i2c_start
		BRES		I2C_DIR, #SDA
		BRES		I2C_DIR, #SCL
		BSET		I2C_DIR, #SDA
		CALL		i2c_delay
		BSET		I2C_DIR, #SCL
		RET
		
;------------------------------------------------------------------------------
; STOP routine
;------------------------------------------------------------------------------
.i2c_stop
		BRES		I2C_DIR, #SCL
		CALL		i2c_delay
		BRES		I2C_DIR, #SDA
		RET
		
;------------------------------------------------------------------------------
; Write byte to I2C bus
; ACK bit is return in C flag
; Byte to send must be into A register before call this routine
;------------------------------------------------------------------------------
.i2c_write
		PUSH		A
		LD			i2c_data, A 					; store A into i2c_data
		LD			A, #8 								; 
		LD			i2c_counter, A 				; loop counter
i2cw1
		BSET		I2C_DIR, #SCL					; 
		BSET		I2C_DIR, #SDA					; 
		LD			A, i2c_data						; restore data
		RLC			A											; rotate left through C flag
		LD			i2c_data, A						; store data
		JRNC		i2cw2									; branch if not carry
		BRES		I2C_DIR, #SDA					; else SDA go high
i2cw2
		BRES		I2C_DIR, #SCL 				; set SCL
		DEC			i2c_counter 					; decrease loop counter
		JRNE		i2cw1 								; jump if greater than 0
		BSET		I2C_DIR, #SCL 				; clear SCL
		BRES		I2C_DIR, #SDA					; set SDA
		BRES		I2C_DIR, #SCL					; set SCL
		BTJT		I2C_DAT, #SDA, i2cw3 	; copy SDA into C (read ACK bit)
i2cw3		
		BSET		I2C_DIR, #SDA					; clear SDA
		BSET		I2C_DIR, #SCL					; clear SCL
		POP			A
		RET

;------------------------------------------------------------------------------
; Read byte from I2C bus
; ACK bit must be written into C flag before call this routine
; Read byte is return in A register
;------------------------------------------------------------------------------
.i2c_read
		RLC			A											; store C flag
		LD			i2c_data, A						;
		LD			A,#8									;
		LD 			i2c_counter, A 				; loop counter
		BRES		I2C_DIR, #SDA					; set SDA
i2cr1
		CALL 		i2c_delay 						; wait some usecs
		BRES		I2C_DIR, #SCL   			; set sCL
		BTJT		I2C_DAT, #SDA, i2cr2	; copy SDA into C (read data bit)
i2cr2
		LD			A, i2c_data 					; restore data
		RLC 		A 										; store new bit
		LD			i2c_data, A 					; store data
		BSET		I2C_DIR, #SCL					; niski stan na linii SCL
		DEC			i2c_counter 					; decrease loop counter
		JRNE		i2cr1 								; jump, if greater than zero
		BSET		I2C_DIR, #SDA  				; clear SDA
		JRNC		i2cr3 								; jump, if C flag is clear
		BRES		I2C_DIR, #SDA   			; set SDA
i2cr3		
		BRES		I2C_DIR, #SCL					; set SCL
		BSET		I2C_DIR, #SCL					; clear SCL
		BSET		I2C_DIR, #SDA 				; clear SDA
		LD			A, i2c_data 					; move read byte to A register
		RET

;------------------------------------------------------------------------------
; End of I2C.asm
;------------------------------------------------------------------------------
		END
		